@@ -8,17 +8,17 @@ module System.FSNotify.OSX
        , NativeManager
        ) where
 
-import Prelude hiding (FilePath, catch)
+import Prelude hiding (FilePath)
 
 import Control.Concurrent.Chan
 import Control.Concurrent.MVar
-import Control.Monad hiding (void)
+import Control.Monad
 import Data.Bits
 import Data.IORef (atomicModifyIORef, readIORef)
 import Data.Map (Map)
 import Data.Time.Clock (UTCTime, getCurrentTime)
 import Data.Word
--- import Debug.Trace (trace)
+import Data.Unique
 import Filesystem (isFile)
 import Filesystem.Path hiding (concat)
 import System.FSNotify.Listener
@@ -30,15 +30,10 @@ import qualified System.OSX.FSEvents as FSE
 data ListenType = NonRecursive | Recursive
 data WatchData = WatchData FSE.EventStream ListenType EventChannel
 
--- TODO: We really should use something other than FilePath as a key to allow
--- for more than one listener per FilePath.
-type WatchMap = Map FilePath WatchData
+type WatchMap = Map Unique WatchData
 data OSXManager = OSXManager (MVar WatchMap)
 type NativeManager = OSXManager
 
-void :: IO ()
-void = return ()
-
 nil :: Word64
 nil = 0x00
 
@@ -72,17 +67,11 @@ fsnEvents timestamp fseEvent = liftM concat . sequence $ map (\f -> f fseEvent)
 -- hfsevents package doesn't support non-recursive event reporting.
 
 handleNonRecursiveFSEEvent :: ActionPredicate -> EventChannel -> FilePath -> DebouncePayload -> FSE.Event -> IO ()
--- handleNonRecursiveFSEEvent _       _    dirPath _   fseEvent | trace ("OSX: handleNonRecursiveFSEEvent " ++ show dirPath ++ " " ++ show fseEvent) False = undefined
 handleNonRecursiveFSEEvent actPred chan dirPath dbp fseEvent = do
   currentTime <- getCurrentTime
   events <- fsnEvents currentTime fseEvent
   handleNonRecursiveEvents actPred chan dirPath dbp events
 handleNonRecursiveEvents :: ActionPredicate -> EventChannel -> FilePath -> DebouncePayload -> [Event] -> IO ()
--- handleNonRecursiveEvents actPred _    dirPath _   (event:_     ) | trace (   "OSX: handleNonRecursiveEvents "
---                                                                       ++ show dirPath ++ " " ++ show event
---                                                                       ++ "\n  " ++ fp (directory dirPath)
---                                                                       ++ "\n  " ++ fp (directory (eventPath event))
---                                                                       ++ "\n  " ++ show (actPred event)) False = undefined
 handleNonRecursiveEvents actPred chan dirPath dbp (event:events)
   | directory dirPath == directory (eventPath event) && actPred event = do
     case dbp of
@@ -93,17 +82,15 @@ handleNonRecursiveEvents actPred chan dirPath dbp (event:events)
       Nothing                           -> writeChan chan event
     handleNonRecursiveEvents actPred chan dirPath dbp events
   | otherwise                                                         = handleNonRecursiveEvents actPred chan dirPath dbp events
-handleNonRecursiveEvents _ _ _ _ []                                   = void
+handleNonRecursiveEvents _ _ _ _ []                                   = return ()
 
 handleFSEEvent :: ActionPredicate -> EventChannel -> DebouncePayload -> FSE.Event -> IO ()
--- handleFSEEvent _       _    _   fseEvent | trace ("OSX: handleFSEEvent " ++ show fseEvent) False = undefined
 handleFSEEvent actPred chan dbp fseEvent = do
   currentTime <- getCurrentTime
   events <- fsnEvents currentTime fseEvent
   handleEvents actPred chan dbp events
 
 handleEvents :: ActionPredicate -> EventChannel -> DebouncePayload -> [Event] -> IO ()
--- handleEvents actPred _    _   (event:_     ) | trace ("OSX: handleEvents " ++ show event ++ " " ++ show (actPred event)) False = undefined
 handleEvents actPred chan dbp (event:events) = do
   when (actPred event) $ case dbp of
       (Just (DebounceData epsilon ior)) -> do
@@ -112,7 +99,25 @@ handleEvents actPred chan dbp (event:events) = do
         atomicModifyIORef ior (\_ -> (event, ()))
       Nothing                           -> writeChan chan event
   handleEvents actPred chan dbp events
-handleEvents _ _ _ [] = void
+handleEvents _ _ _ [] = return ()
+
+listenFn
+  :: (ActionPredicate -> EventChannel -> FilePath -> DebouncePayload -> FSE.Event -> IO a)
+  -> WatchConfig
+  -> OSXManager
+  -> FilePath
+  -> ActionPredicate
+  -> EventChannel
+  -> IO StopListening
+listenFn handler conf (OSXManager mvarMap) path actPred chan = do
+  path' <- canonicalizeDirPath path
+  dbp <- newDebouncePayload $ confDebounce conf
+  unique <- newUnique
+  eventStream <- FSE.eventStreamCreate [fp path'] 0.0 True False True (handler actPred chan path' dbp)
+  modifyMVar_ mvarMap $ \watchMap -> return (Map.insert unique (WatchData eventStream NonRecursive chan) watchMap)
+  return $ do
+    FSE.eventStreamDestroy eventStream
+    modifyMVar_ mvarMap $ \watchMap -> return $ Map.delete unique watchMap
 
 instance FileListener OSXManager where
   initSession = do
@@ -127,20 +132,8 @@ instance FileListener OSXManager where
       eventStreamDestroy' :: WatchData -> IO ()
       eventStreamDestroy' (WatchData eventStream _ _) = FSE.eventStreamDestroy eventStream
 
-  listen db (OSXManager mvarMap) path actPred chan = do
-    path' <- canonicalizeDirPath path
-    dbp <- newDebouncePayload db
-    eventStream <- FSE.eventStreamCreate [fp path'] 0.0 True False True (handler path' dbp)
-    modifyMVar_ mvarMap $ \watchMap -> return (Map.insert path' (WatchData eventStream NonRecursive chan) watchMap)
-    where
-      handler :: FilePath -> DebouncePayload -> FSE.Event -> IO ()
-      handler = handleNonRecursiveFSEEvent actPred chan
+  listen = listenFn handleNonRecursiveFSEEvent
 
-  listenRecursive db (OSXManager mvarMap) path actPred chan = do
-    path' <- canonicalizeDirPath path
-    dbp <- newDebouncePayload db
-    eventStream <- FSE.eventStreamCreate [fp path'] 0.0 True False True $ handler dbp
-    modifyMVar_ mvarMap $ \watchMap -> return (Map.insert path' (WatchData eventStream Recursive chan) watchMap)
-    where
-      handler :: DebouncePayload -> FSE.Event -> IO ()
-      handler = handleFSEEvent actPred chan
+  listenRecursive = listenFn $ \actPred chan path -> handleFSEEvent actPred chan
+
+  usesPolling = const False
